/* RXEPI1.CMD  */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      RXEPI1.CMD                                     */
/*                                                                 */
/* DESCRIPTIVE NAME Demonstrates RxEPI                             */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1995.                  */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 4 Mod 3                      */
/* Modified         Sept 6th 1995                                  */
/*                  Corrections by Bill Matthews/Jeff Hooper       */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 V2.x,CICS OS/2, RxEPI                    */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/* Sample program demonstrates the usage of the RXEPI by           */
/* starting transaction CESN on a new terminal and quitting it     */
/* again.                                                          */
/*******************************************************************/
SIGNAL ON FAILURE  NAME FAILURE
SIGNAL ON ERROR    NAME FAILURE
SIGNAL ON NOTREADY NAME FAILURE
SIGNAL ON SYNTAX  NAME FAILURE
trace off
/*
   Use DEBUG to display RXEPI internal information: 0 - 5
   can be changed between different functions
   Normally, the information goes to stderr, specifying a DEBUGFILE
   appends the information to that file.
   You must EXIT the OS/2 session in order to access the DEBUFILE !!!
*/
/*DEBUG=5*/
/*DEBUGFILE="rxepi.log"*/

/* add the Epi functionality as external function */
    if rxfuncquery('RxEpiLoad') Then Do
    call RxFuncDrop 'RxEpiLoad'
    call RxFuncAdd 'RxEpiLoad', 'RXEPI', 'RxEpiLoad'
    call RxEpiLoad
    Say 'RxEpiLoad is complete.'
    end

/*******************************************************************/
/* initialize  EPI                                                 */
/*******************************************************************/
say "Initialize EPI"
rc = RxEpiInitialize()
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiInitialize", rc, TERMINDEX);
        end
/*******************************************************************/
/* list available systems                                          */
/*******************************************************************/
say "ListSystems"
rc = RxEpiListSystems('','SYSTEMS','LIST.')
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiListSystems", rc, TERMINDEX);
        end
say "Number of Systems:" SYSTEMS
say LIST._SYSTEMNAME LIST._DESCRIPTION

/*******************************************************************/
/* add a terminal                                                  */
/*******************************************************************/
say "Adding Terminal"
/*
rc = RxEpiAddTerminal('',LIST._SYSTEMNAME,'V124','','','DETAILS.','TERMINDEX')
rc = RxEpiAddTerminal('',LIST._SYSTEMNAME,'','DFLT','','DETAILS.','TERMINDEX')
*/
rc = RxEpiAddTerminal('',LIST._SYSTEMNAME,'','','','DETAILS.','TERMINDEX')
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiAddTerminal", rc, TERMINDEX);
        end
say "TermIndex:" TERMINDEX
say "Details:"
/*Next line removed for compatibility with CICS Clients - JH 6 Sep 95 */
/*say "Termname" DETAILS._TERMNAME "Netname" DETAILS._NETNAME */
say "Numlines" DETAILS._NUMLINES "Numcolumns" DETAILS._NUMCOLUMNS
say "MaxData" DETAILS._MAXDATA
say "Various Flags:"
say DETAILS._ERRLASTLINE DETAILS._ERRINTENSIFY DETAILS._ERRCOLOR
say DETAILS._ERRHILIGHT  DETAILS._HILIGHT      DETAILS._COLOR
say DETAILS._PRINTER

/*******************************************************************/
/* getting ATI state                                               */
/*******************************************************************/

say "Getting ATI State"
STATE = 'CICS_EPI_ATI_QUERY'
rc = RxEpiATIState(TERMINDEX,'STATE')
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiATIState", rc, TERMINDEX);
        end
say "ATI State is" STATE

/*******************************************************************/
/* start transaction                                               */
/*******************************************************************/

say "Starting Transaction"
DATA = '27'x || '20'x || '20'x
DATA = DATA || 'CESN'
say DATA
DATASIZE = LENGTH(DATA)
say DATASIZE
rc = RxEpiStartTran(TERMINDEX,'', DATA, DATASIZE)
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiStartTran", rc, TERMINDEX);
        end

/*******************************************************************/
/* event processing                                                */
/*******************************************************************/

do forever
    say "Getting Event"
    EVENT._SIZE = 500
    rc = RxEpiGetEvent(TERMINDEX,'CICS_EPI_WAIT','EVENT.')
    say "rc =" rc
    if rc \= 'CICS_EPI_NORMAL' & rc \= 'CICS_EPI_ERR_MORE_EVENTS' then do
        rc = ErrorExit("RxEpiGetEvent", rc, TERMINDEX)
        end
    say "TERMINDEX"  EVENT._TERMINDEX
    say "EVENT    "  EVENT._EVENT
    say "ENDREASON"  EVENT._ENDREASON
    say "TRANSID  "  EVENT._TRANSID
    say "ABENDCODE"  EVENT._ABENDCODE
    say "SIZE     "  EVENT._SIZE
    Select
        when EVENT._EVENT = "CICS_EPI_EVENT_DUMMY" then do
             say "handling CICS_EPI_EVENT_DUMMY"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_SEND" then do
             say "handling CICS_EPI_EVENT_SEND"
             rc = display_data(EVENT._DATA)
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_CONVERSE" then do
             say "handling CICS_EPI_EVENT_CONVERSE"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TRAN" then do
             say "handling CICS_EPI_EVENT_END_TRAN"
             if EVENT._TRANSID = '' then do
                say "leaving first loop"
                leave
                end
             else do
                say "quitting transaction"
                /* send back a PF3 */
                DATA = '33'x || '20'x || '20'x
                DATA = DATA || EVENT._TRANSID
                say DATA
                DATASIZE = LENGTH(DATA)
                say DATASIZE
                rc = RxEpiStartTran(TERMINDEX,'', DATA, DATASIZE)
                if rc \= 'CICS_EPI_NORMAL' then do
                   rc = ErrorExit("RxEpiStartTran", rc, TERMINDEX);
                   end
                end
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_START_ATI" then do
             say "handling CICS_EPI_EVENT_START_ATI"
        end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TERM" then do
             say "handling CICS_EPI_EVENT_END_TERM"
             leave
        end
        otherwise say "Unknown Event"
    end
end

/*******************************************************************/
/* Delete Terminal                                                 */
/*******************************************************************/
say "Deleting Terminal"
rc = RxEpiDelTerminal(TERMINDEX)
if rc \= 'CICS_EPI_NORMAL' then do
    rc = ErrorExit("RxEpiDelTerminal", rc, TERMINDEX);
    end

/*******************************************************************/
/* event processing                                                */
/*******************************************************************/
do forever
    say "Getting Event"
    EVENT._SIZE = 500
    rc = RxEpiGetEvent(TERMINDEX,'CICS_EPI_WAIT','EVENT.')
    say "rc =" rc
    if rc \= 'CICS_EPI_NORMAL' & rc \= 'CICS_EPI_ERR_MORE_EVENTS' then do
        rc = ErrorExit("RxEpiGetEvent", rc, TERMINDEX)
        end
    say "TERMINDEX"  EVENT._TERMINDEX
    say "EVENT    "  EVENT._EVENT
    say "ENDREASON"  EVENT._ENDREASON
    say "TRANSID  "  EVENT._TRANSID
    say "ABENDCODE"  EVENT._ABENDCODE
    say "SIZE     "  EVENT._SIZE
    Select
        when EVENT._EVENT = "CICS_EPI_EVENT_DUMMY" then do
             say "handling CICS_EPI_EVENT_DUMMY"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_SEND" then do
             say "handling CICS_EPI_EVENT_SEND"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_CONVERSE" then do
             say "handling CICS_EPI_EVENT_CONVERSE"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TRAN" then do
             say "handling CICS_EPI_EVENT_END_TRAN"
             end
        when EVENT._EVENT = "CICS_EPI_EVENT_START_ATI" then do
             say "handling CICS_EPI_EVENT_START_ATI"
        end
        when EVENT._EVENT = "CICS_EPI_EVENT_END_TERM" then do
             say "handling CICS_EPI_EVENT_END_TERM"
             say "leaving second loop"
             leave
        end
        otherwise say "Unknown Event"
    end
end
/*******************************************************************/
/* terminating                                                     */
/*******************************************************************/
say "Terminating"
rc = RxEpiTerminate()
if rc \= 'CICS_EPI_NORMAL' then do
        rc = ErrorExit("RxEpiTerminate", rc, TERMINDEX);
        end

/*******************************************************************/
/* just a test                                                     */
/*******************************************************************/

rc = RxEpiGetSysError(TermIndex, 'SYSERR.')
say "Cause:" SYSERR._Cause
say "Error:" SYSERR._Value
say "Message:" SYSERR._Msg

/*******************************************************************/
/* dropping functions, can be omitted                              */
/*******************************************************************/
Call RxEpiDrop
Say 'The functions have now been dropped.'
return 0

/*****************************************************************/
/*****************************************************************/
/*****************************************************************/
/*                 Some utility functions                        */
/*****************************************************************/
/*****************************************************************/
/*****************************************************************/


 /*********************************************************************/
 /* display_data()                                                    */
 /*********************************************************************/
display_data: procedure
arg DATA
   say "DATA"
   say DATA
return(0)

 /*********************************************************************/
 /* ErrorExit() - Report errors and exit                              */
 /*********************************************************************/
ErrorExit: procedure
arg function, rc, TERMINDEX

   say function || "() call failed "
   if (rc \= "CICS_EPI_ERR_FAILED") then do
      say "with return code message:" rc
      end
   else do
                                       /* EPI_ERR_FAILED, more error  */
                                       /* information in GetSysError  */
      rc = RxEpiGetSysError(TermIndex, 'SYSERR.');
      say "with a system error"
      say "Cause:" SYSERR._Cause
      say "Error:" SYSERR._Value
      say "Message:" SYSERR._Msg
      end
rc = RxEpiTerminate()
exit
return (0)

 /*********************************************************************/
 /* Failure()                                                         */
 /*********************************************************************/
FAILURE:
  Say 'Failure line' sigl
  Say '+++' sourceline(sigl)
rc = RxEpiTerminate()
  exit
