/* CALLPA2.CMD  */
/*******************************************************************/
/* MODULE NAME      CALLPA2.CMD                                    */
/*                                                                 */
/* DESCRIPTIVE NAME Demonstrates RxECI and CICS OS/2 Performance   */
/*                                                   Analyzer      */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  CP31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1994, 1995             */
/*                  See Copyright Instructions.                    */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 1 Release 4                            */
/* Last Modified    4 April 1995  (Non functional)                 */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 V2.x,CICS OS/2, RxECI, CICS OS/2 PA      */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*******************************************************************/
/* Calls and Decodes the PA2KCBIN function                         */
/*******************************************************************/
'@ECHO OFF'
  Call RexxSetup                        /* Ensure RexxUtil loaded  */
  Call EciSetup                         /* Ensure RxCICS loaded    */
/*-------------------- Debug Option --------------*/
DEBUG=0
DEBUGFILE=""
/*-----------------------------------------------------------------*/
NUMERIC DIGITS 32  /* this is important! for numeric functions     */
/*-----------------------------------------------------------------*/
/*  CALLPA2 choice repeat delay                                    */
/*                                                                 */
/*    choice = single character - what stats are desired           */
/*             default=P for task stats                            */
/*    repeat = how many times to repeat the request                */
/*             default=1 one time (0 is forever)                   */
/*    delay  = amount of time, in seconds, between repeats         */
/*             default=5 seconds                                   */
/*-----------------------------------------------------------------*/
Parse Value 'P 1 5 30 24000' with,
   default_choice default_repeat default_delay choice_position,
   default_commarea_length
/* extracted from the PA2ALLEN.H file */      /*-------------------*/
 PCT_TYPE_RECORD   =    'P'                   /* Only the P, X and */
 PID_TYPE_RECORD   =    'X'                   /* E options are     */
 BUK_TYPE_RECORD   =    'E'                   /* currently         */
 FCT_TYPE_RECORD   =    'F'                   /* implemented       */
 SNT_TYPE_RECORD   =    'S'                   /* by PA2KCBIN.      */
 TCT_TYPE_RECORD   =    'T'                   /*                   */
 SIT_TYPE_RECORD   =    'Z'                   /* ALL OTHER OPTIONS */
 TST_TYPE_RECORD   =    'Q'                   /* have not been     */
 TCT_TYPE_RECORD   =    'T'                   /* tested and may be */
 DCT_TYPE_RECORD   =    'D'                   /* incomplete.       */
 PPT_TYPE_RECORD   =    'O'                   /*                   */
/* Length type definitions          */        /*-------------------*/
 PIDSTACKNO     =   9; NOCLOCKS       =   4; KEYLEN         =  12;
 APPLIDLEN      =   8; FILELEN        =   8; PROGRAMLEN     =   8;
 RESOURCELEN    =   8; GROUPLEN       =   8; TSQLEN         =   8;
 TRANLEN        =   4; ABCODELEN      =   4; SYSIDLEN       =   4;
 TASKNLEN       =   4; TDQLEN         =   4;
/*----------------------------------------------------------------*/
valid_choices= PCT_TYPE_RECORD BUK_TYPE_RECORD FCT_TYPE_RECORD,
               SNT_TYPE_RECORD TCT_TYPE_RECORD SIT_TYPE_RECORD,
               TST_TYPE_RECORD TCT_TYPE_RECORD DCT_TYPE_RECORD,
               PPT_TYPE_RECORD PID_TYPE_RECORD
/*----------------------------*/
/* Get user's input           */
/*----------------------------*/
Arg Choice Repeat Delay Commarea_Length .
/*----------------------------*/
/* validate user's input      */
/*----------------------------*/
If Commarea_Length='' Then Commarea_Length=default_commarea_length
If Delay ='' Then delay =default_delay
If Repeat='' Then repeat=default_repeat
If Choice='' Then Choice=default_choice
If Pos(choice,valid_choices)=0 Then choice=default_choice
/*------------------------------------------------*/
/* Set call parms                                 */
/*------------------------------------------------*/
eci.transid='CPMI'
ECI.userid            = "SYSAD"
ECI.password          = "SYSAD"
ECI.program_name      = "PA2KCBIN"
/*------------------------------------------------*/
/* Repeatable Calls to display stats              */
/*------------------------------------------------*/
Do Repeat_Call = 1 to Repeat
   rc= prime_loop()            /* make the call   */
   If Repeat_Call<> Repeat Then call SysSleep delay
End
If DEBUG>0 Then rc = rxfuncdrop('ECI')  /* Drop when in Debug  */
Return 0
/*----------------------------------------------------*/
/* Primary Loop                                       */
/*----------------------------------------------------*/
prime_loop:
    /*------------------------------------------------*/
    /* provide the Commarea with user's choice        */
    /*------------------------------------------------*/
    commareaI= left(' ',choice_position)||Choice||left(' ',commarea_length)

    If DEBUG>0 Then rc = show_eci_stem()   /* show call options */

    commarea =ECI('ECI.',commareaI)        /* Actual ECI Call   */

    rc = check_rc(commarea2)               /* Check ECI Response*/
    call get_record commarea               /* Decode Response   */
Return 0
/*--------------------------------------------------------------*/
/*   Decode Information  returned by PA2KCBIN                   */
/*--------------------------------------------------------------*/
Get_Record:
  Arg commarea
  Call Get_Table_Header commarea       /* Decode the Header     */
  Type_Record=DAT_PA2TABLE_KEY         /* Type_Record indicates */
  Do_Count=DAT_PA2TABLE_short_noitems  /* Count of Items Returnd*/
  if do_count<1 Then
    Do
      Say 'There is no information to display for choice' choice
      Exit 99
    End
  'CLS'
  say ''                    /* for spacing if in a window       */
  lineH.1=left(date(),12),
          center('CICS OS/2 Performance Analyzer for Sysid=' applid,56),
          right(ftime,8)
  say lineH.1
  /*------------------------------------------------------------*/
  /*  Format individual Responses                               */
  /*------------------------------------------------------------*/
  Select                               /* Format based on Type  */
    When Type_Record = PID_TYPE_RECORD Then  /* Process Stats   */
       Do
         line_type='OS/2 Process Statistics'
         lineH.2=center(line_type,52) right('Total Execution Time='tottime,22)
         say lineH.2
         say ''
         lineH.3='Process Trans ...Time (seconds) OS/2    Tran',
         'Task .. Program......'
         lineH.4='Number  Run   Total    Active   Priorty ID  ',
         'Numb Depth Name......'
         lineH.5='------- ----- -------- -------- ------- ----',
         '---- ----- ----------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
            Call Get_PID_Type_Record commarea
            say line.I
         End
      End
    When Type_Record = PCT_TYPE_RECORD Then     /* Task Stats            */
       Do
         line_type='Task Statistics'
         lineH.2=left('Total Transactions='kctrantot,22),
         center(line_type,22) right('Total Execution Time='tottime,30)
         say lineH.2
         say ''
         lineH.3='Transaction.... ......Time (seconds).......... Abend Sync'||,
                 'points   FilMgrSyncPt'
         lineH.4='  Name   Count  .Total. Average .Short. .Long.. Numb Numb',
                 '.Time.. Numb .Time..'
         lineH.5='-------- ------ ------- ------- ------- ------- ---- ----',
                 '------- ---- -------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
            Call Get_PCT_Type_Record commarea
            say line.I
         End
      End
    When Type_Record = BUK_TYPE_RECORD Then        /* Bucket Stats          */
       Do
         line_type='Clock Statistics'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ............Clock.......................'
         lineH.4='Name  Count Number   Started Stopped seconds Average'
         lineH.5='---- ------ -------- ------- ------- ------- -------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
            Call Get_BUK_Type_Record commarea
            Do K=1 to 4
               say line.K
            End
         End
       End
    When Type_Record = FCT_TYPE_RECORD Then        /* File Stats            */
       Do
         line_type='File Statistics'
         lineH.2= center(line_type,78)
         Do I=1 to Do_Count
           Call Get_FCT_Type_Record commarea
           say lineH.2
           say ''
           Do Lin=1 to 20
              say L.Lin
           End
         End
         Do Lin=1 to 20
            say L.Lin
         End
       End
    When Type_Record = SNT_TYPE_RECORD Then        /* SNT Stats             */
       Do
         line_type='SignOn Table Information'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
           Call Get_SNT_Type_Record commarea
         End
       End
    When Type_Record = TCT_TYPE_RECORD Then        /* Terminal Stats        */
       Do
         line_type='Terminal Statistics'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
           Call Get_TCT_Type_Record commarea
         End
       End
    When Type_Record = SIT_TYPE_RECORD Then        /* SIT Information       */
      Do
         line_type='System Initialization Information'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
           Call Get_SIT_Type_Record commarea
         End
       End
    When Type_Record = TST_TYPE_RECORD Then        /* Temp Storage Stats    */
       Do
         line_type='Temporary Storage Statistics'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
           Call Get_TST_Type_Record commarea
         End
       End
    When Type_Record = DCT_TYPE_RECORD Then        /* Tran Data Stats       */
       Do
         line_type='Transient Data Statistics'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
           Call Get_DCT_Type_Record commarea
         End
       End
    When Type_Record = PPT_TYPE_RECORD Then        /* PPT Stats             */
       Do
         line_type='Program Statistics'
         lineH.2= center(line_type,78)
         say lineH.2
         say ''
         lineH.3='Transaction ........................................'
         lineH.4='Name                                                '
         lineH.5='-------- -------------------------------------------'
         say lineH.3
         say lineH.4
         say lineH.5
         Do I=1 to Do_Count
            Call Get_PPT_Type_Record commarea
         End
       End
    Otherwise
      Do
        Call Error 23 'Invalid Type Record,' type_record
      End
  End
Return
/************************\
* Decode Data Structures *
\************************/
/*******************\
*   PA2TABLE_AREA   *
\*******************/
Get_Table_Header:
  Arg commarea
  Parse Value commarea with 1 PA2TABLE +48 COMMAREA
  Parse Value PA2TABLE with,
       DAT_PA2TABLE_char_applid      +8,      /* Applid from SIT           */
       DAT_PA2TABLE_ulong_kctrantot  +4,      /* total transactions run    */
       DAT_PA2TABLE_ulong_tottime    +4,      /* total run time in seconds */
       DAT_PA2TABLE_char_date        +4,      /* current date - packed     */
       DAT_PA2TABLE_char_time        +4,      /* time of run    packed     */
       DAT_PA2TABLE_short_length     +2,      /* Commarea Length           */
       DAT_PA2TABLE_short_noitems    +2,      /* Number of Entries         */
       DAT_PA2TABLE_short_maxcurrenttran +2,  /* Max concurrent trans      */
       DAT_PA2TABLE_KEY              +1,      /* The key to the rest       */
       DAT_PA2TABLE_char_end         +1,      /* Y if data returned        */
       DAT_PA2TABLE_reserved        +16,      /* Reserved space            */
       .                                      /* unused                    */
      Drop PA2TABLE                           /* no longer needed          */
       DAT_PA2TABLE_char_date      = unpack(DAT_PA2TABLE_char_date)
       DAT_PA2TABLE_char_time      = unpack(DAT_PA2TABLE_char_time)
       DAT_PA2TABLE_short_length   = Z2D(DAT_PA2TABLE_short_length)
       DAT_PA2TABLE_short_noitems  = Z2D(DAT_PA2TABLE_short_noitems)
       DAT_PA2TABLE_short_maxcurrenttran = Z2D(DAT_PA2TABLE_short_maxcurrenttran)
       DAT_PA2TABLE_ulong_kctrantot= Z2D(DAT_PA2TABLE_ulong_kctrantot)
       DAT_PA2TABLE_ulong_tottime  = Z2D(DAT_PA2TABLE_ulong_tottime)

  applid   = DAT_PA2TABLE_char_applid         /* Applid from SIT          */
  date     = DAT_PA2TABLE_char_date           /* current date             */
  fdate    = substr(date,3,2)'/'substr(date,5,3)
  time     = DAT_PA2TABLE_char_time           /* time of run              */
  time     = right('00'||time,6)
  ftime    = left(time,2)':'substr(time,3,2)':'right(time,2)
  maxcurtran=DAT_PA2TABLE_short_maxcurrenttran
  kctrantot= DAT_PA2TABLE_ulong_kctrantot
  tottime  = DAT_PA2TABLE_ulong_tottime
Return
/*************\
*   KEY_SIT   *           /*--------------- undefined -------------*/
\*************/
Get_SIT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_SIT_char_filler   +12,
     DAT_SIT_char_filler
Return
/*************\
*   KEY_PCT   *
\*************/
Get_PCT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_PCT_char_key                  +4, /* Transaction id                */
     .                                 +8, /* Filler                        */
     DAT_PCT_ulong_kctran              +4, /*Number of transactions         */
     DAT_PCT_ulong_kctrantotaltime     +4, /*Sum of resp. times in ms       */
     DAT_PCT_ulong_kctranshortesttime  +4, /*Shortest resp. time in ms      */
     DAT_PCT_ulong_kctranlongesttime   +4, /*Longest resp. time in ms       */
     DAT_PCT_ulong_kcnoabend           +4, /*Number of ABENDs               */
     DAT_PCT_ulong_kcsync              +4, /*Number of SYNCPOINTs           */
     DAT_PCT_ulong_kcsynctotaltime     +4, /*Total SYNCPOINT time, ms       */
     DAT_PCT_ulong_kcbtsync            +4, /*Num of System File Manager calls in SYNCPOINT   */
     DAT_PCT_ulong_kcsyncbttotaltime   +4, /*Total time in System File Manager SYNCPOINT in ms  */
     DAT_PCT_double_kcsq               +8, /* Sum of squares of resp. times in millisecs*/
     .                                     /* end of data area              */
  tranid             = KEY_PCT_char_key
  kcsq               = DAT_PCT_double_kcsq
  kctran             = z2d(DAT_PCT_ulong_kctran)
  kcTranTotalTime    = ftime(z2d(DAT_PCT_ulong_kctrantotaltime))
  kcTranAvgTime      = ftime(trunc((z2d(DAT_PCT_ulong_kctrantotaltime))/kctran))
  kcTranShortestTime = ftime(z2d(DAT_PCT_ulong_kctranshortesttime))
  kcTranLongestTime  = ftime(z2d(DAT_PCT_ulong_kctranlongesttime))
  kcNoAbend          = z2d(DAT_PCT_ulong_kcnoabend)
  kcSync             = z2d(DAT_PCT_ulong_kcsync)
  kcSyncTotalTime    = ftime(z2d(DAT_PCT_ulong_kcsynctotaltime))
  kcBtSync           = z2d(DAT_PCT_ulong_kcbtsync)
  kcSyncBtTotalTime  = ftime(z2d(DAT_PCT_ulong_kcsyncbttotaltime))
  Line.I=center(tranid,8) center(kctran,6) right(kcTranTotalTime,7),
         right(kcTranAvgTime,7) right(kcTranShortestTime,7),
         right(kcTranLongestTime,7) center(kcNoAbend,4),
         center(kcSync,4) right(kcSyncTotalTime,7),
         center(kcBtSync,4) right(kcSyncBtTotalTime,7)
Return
/*************\
*   KEY_BUK   *
\*************/
Get_BUK_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_BUK_char_key                  +4, /* Transaction id          */
     .                                 +8, /* Filler                  */
     DAT_BUK_ulong_kctran              +4, /*Number of transactions   */
     DAT_BUK_ulong_kcbucketno.1        +4, /*bucket[I] started        */
     DAT_BUK_ulong_kcbucketno.2        +4, /*bucket[I] started        */
     DAT_BUK_ulong_kcbucketno.3        +4, /*bucket[I] started        */
     DAT_BUK_ulong_kcbucketno.4        +4, /*bucket[I] started        */
     DAT_BUK_ulong_kcbucketstopno.1    +4, /*bucket[I] stopped        */
     DAT_BUK_ulong_kcbucketstopno.2    +4, /*bucket[I] stopped        */
     DAT_BUK_ulong_kcbucketstopno.3    +4, /*bucket[I] stopped        */
     DAT_BUK_ulong_kcbucketstopno.4    +4, /*bucket[I] stopped        */
     DAT_BUK_ulong_kcbuckettot.1       +4, /*tot time in bucket[I]    */
     DAT_BUK_ulong_kcbuckettot.2       +4, /*tot time in bucket[I]    */
     DAT_BUK_ulong_kcbuckettot.3       +4, /*tot time in bucket[I]    */
     DAT_BUK_ulong_kcbuckettot.4       +5, /*tot time in bucket[I]    */
     .                                     /*end of data area         */
  tranid = KEY_BUK_char_key
  kctran = z2d(DAT_BUK_ulong_kctran)
  Do K=1 to 4
     kcbucketno.K     = z2d(DAT_BUK_ulong_kcbucketno.K)
     kcbucketstopno.K = z2d(DAT_BUK_ulong_kcbucketstopno.K)
     kcbuckettot.K    = ftime(z2d(DAT_BUK_ulong_kcbuckettot.K))
     kcbucketavg.K    = ftime(trunc((z2d(DAT_BUK_ulong_kcbuckettot.K))/kctran))
     Line.K=left(tranid,4) right(kctran,6),
            right('CLOCK('k')',8) right(kcbucketno.K,7),
            right(kcbucketstopno.K,7) right(kcbuckettot.K,7),
            right(kcbucketavg.K,7)
  End
Return
/*************\
*   KEY_FCT   *
\*************/
Get_FCT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_FCT_char_key            +8, /* File name                    */
     KEY_FCT_char_tranid         +4, /* Tran id                      */
     DAT_FCT_ulong_fctran        +4, /* times transaction ran        */
     DAT_FCT_ulong_fcread        +4, /* Number of READs              */
     DAT_FCT_ulong_fcwrite       +4, /* Number of WRITEs             */
     DAT_FCT_ulong_fcrewrite     +4, /* Number of REWRITEs           */
     DAT_FCT_ulong_fcdelete      +4, /* Number of DELETEs            */
     DAT_FCT_ulong_fcunlock      +4, /* Number of UNLOCKs            */
     DAT_FCT_ulong_fcstartbr     +4, /* Number of STARTBRowses       */
     DAT_FCT_ulong_fcresetbr     +4, /* Number of RESETBRowses       */
     DAT_FCT_ulong_fcendbr       +4, /* Number of ENDBRowses         */
     DAT_FCT_ulong_fcreadnext    +4, /* Number of READNEXTs          */
     DAT_FCT_ulong_fcreadprev    +4, /* Number of READPREVious       */
     DAT_FCT_ulong_fcbtread      +4, /* Calls for READs              */
     DAT_FCT_ulong_fcbtwrite     +4, /* Calls for WRITEs             */
     DAT_FCT_ulong_fcbtrewrite   +4, /* Calls for REWRITEs           */
     DAT_FCT_ulong_fcbtdelete    +4, /* Calls for DELETEs            */
     DAT_FCT_ulong_fcbtunlock    +4, /* Calls for UNLOCKs            */
     DAT_FCT_ulong_fcbtstartbr   +4, /* Calls for STARTBRowses       */
     DAT_FCT_ulong_fcbtresetbr   +4, /* Calls for RESETBRowses       */
     DAT_FCT_ulong_fcbtendbr     +4, /* Calls for ENDBRowses         */
     DAT_FCT_ulong_fcbtreadnext  +4, /* Calls for READNEXTs          */
     DAT_FCT_ulong_fcbtreadprev  +4, /* Calls for READPREVious       */
     DAT_FCT_ulong_fctotread     +4, /* For READs                    */
     DAT_FCT_ulong_fctotwrite    +4, /* For WRITEs                   */
     DAT_FCT_ulong_fctotrewrite  +4, /* For REWRITEs                 */
     DAT_FCT_ulong_fctotdelete   +4, /* For DELETEs                  */
     DAT_FCT_ulong_fctotunlock   +4, /* For UNLOCKs                  */
     DAT_FCT_ulong_fctotstartbr  +4, /* For STARTBRowses             */
     DAT_FCT_ulong_fctotresetbr  +4, /* For RESETBRowses             */
     DAT_FCT_ulong_fctotendbr    +4, /* For ENDBRowses               */
     DAT_FCT_ulong_fctotreadnext +4, /* For READNEXTs                */
     DAT_FCT_ulong_fctotreadprev +4, /* For READPREVious             */
     .                               /* end of data                  */

     key          =    KEY_FCT_char_key           )       /* File name              */
     tranid       =    KEY_FCT_char_tranid        )       /* Tran id                */
                   /*--------------------------------------- Number of:---*/
     fctran       =Z2D(DAT_FCT_ulong_fctran       )       /* times transaction run  */
     fcread       =Z2D(DAT_FCT_ulong_fcread       )       /* READs        */
     fcwrite      =Z2D(DAT_FCT_ulong_fcwrite      )       /* WRITEs       */
     fcrewrite    =Z2D(DAT_FCT_ulong_fcrewrite    )       /* REWRITEs     */
     fcdelete     =Z2D(DAT_FCT_ulong_fcdelete     )       /* DELETEs      */
     fcunlock     =Z2D(DAT_FCT_ulong_fcunlock     )       /* UNLOCKs      */
     fcstartbr    =Z2D(DAT_FCT_ulong_fcstartbr    )       /* STARTBRowses */
     fcresetbr    =Z2D(DAT_FCT_ulong_fcresetbr    )       /* RESETBRowses */
     fcendbr      =Z2D(DAT_FCT_ulong_fcendbr      )       /* ENDBRowses   */
     fcreadnext   =Z2D(DAT_FCT_ulong_fcreadnext   )       /* READNEXTs    */
     fcreadprev   =Z2D(DAT_FCT_ulong_fcreadprev   )       /* READPREVious */
                   /*----------Number of System File Manager calls for:-- */
     fcbtread     =Z2D(DAT_FCT_ulong_fcbtread     )       /* READs        */
     fcbtwrite    =Z2D(DAT_FCT_ulong_fcbtwrite    )       /* WRITEs       */
     fcbtrewrite  =Z2D(DAT_FCT_ulong_fcbtrewrite  )       /* REWRITEs     */
     fcbtdelete   =Z2D(DAT_FCT_ulong_fcbtdelete   )       /* DELETEs      */
     fcbtunlock   =Z2D(DAT_FCT_ulong_fcbtunlock   )       /* UNLOCKs      */
     fcbtstartbr  =Z2D(DAT_FCT_ulong_fcbtstartbr  )       /* STARTBRowses */
     fcbtresetbr  =Z2D(DAT_FCT_ulong_fcbtresetbr  )       /* RESETBRowses */
     fcbtendbr    =Z2D(DAT_FCT_ulong_fcbtendbr    )       /* ENDBRowses   */
     fcbtreadnext =Z2D(DAT_FCT_ulong_fcbtreadnext )       /* READNEXTs    */
     fcbtreadprev =Z2D(DAT_FCT_ulong_fcbtreadprev )       /* READPREVious */
                   /*-------------------- Total Time in millisecs for:----*/
     fctotread    =ftime(Z2D(DAT_FCT_ulong_fctotread    ))/* READs        */
     fctotwrite   =ftime(Z2D(DAT_FCT_ulong_fctotwrite   ))/* WRITEs       */
     fctotrewrite =ftime(Z2D(DAT_FCT_ulong_fctotrewrite ))/* REWRITEs     */
     fctotdelete  =ftime(Z2D(DAT_FCT_ulong_fctotdelete  ))/* DELETEs      */
     fctotunlock  =ftime(Z2D(DAT_FCT_ulong_fctotunlock  ))/* UNLOCKs      */
     fctotstartbr =ftime(Z2D(DAT_FCT_ulong_fctotstartbr ))/* STARTBRowses */
     fctotresetbr =ftime(Z2D(DAT_FCT_ulong_fctotresetbr ))/* RESETBRowses */
     fctotendbr   =ftime(Z2D(DAT_FCT_ulong_fctotendbr   ))/* ENDBRowses   */
     fctotreadnext=ftime(Z2D(DAT_FCT_ulong_fctotreadnext))/* READNEXTs    */
     fctotreadprev=ftime(Z2D(DAT_FCT_ulong_fctotreadprev))/* READPREVious */

     /*---------- format for display -----------------*/
     L.1 ='File Name:' right(key,8) 'TranID:' right(tranid,4) 'Trans Count:' right(fctran,7)
     L.2 =''
     L.3 ='Transa- System          Transa- System          Transa- System         '
     L.4 ='ction   File    Total   ction   File    Total   ction   File    Total  '
     L.5 ='calls   Manager Time    calls   Manager Time    calls   Manager Time   '
     L.6 ='Read                    Write                   ReWrite                '
     L.7 ='------- ------- ------- ------- ------- ------- ------- ------- -------'
     L.8 = Right(fcread       ,7) Right(fcbtread     ,7) Right(fctotread    ,7),
           Right(fcwrite      ,7) Right(fcbtwrite    ,7) Right(fctotwrite   ,7),
           Right(fcrewrite    ,7) Right(fcbtrewrite  ,7) Right(fctotrewrite ,7)
     L.9 =''
     L.10='Delete                  Unlock                  StartBrowse            '
     L.11='------- ------- ------- ------- ------- ------- ------- ------- -------'
           Right(fcdelete     ,7) Right(fcbtdelete   ,7) Right(fctotdelete  ,7),
           Right(fcunlock     ,7) Right(fcbtunlock   ,7) Right(fctotunlock  ,7),
           Right(fcstartbr    ,7) Right(fcbtstartbr  ,7) Right(fctotstartbr ,7)
     L.12=''
     L.13='ResetBrowse             EndBrowse               ReadNext               '
     L.14='------- ------- ------- ------- ------- ------- ------- ------- -------'
     L.15= Right(fcresetbr    ,7) Right(fcbtresetbr  ,7) Right(fctotresetbr ,7),
           Right(fcendbr      ,7) Right(fcbtendbr    ,7) Right(fctotendbr   ,7),
           Right(fcreadnext   ,7) Right(fcbtreadnext ,7) Right(fctotreadnext,7)
     L.16=''
     L.17='ReadPrevious            '
     L.18='------- ------- ------- '
     L.19= Right(fcreadprev   ,7) Right(fcbtreadprev ,7) Right(fctotreadprev,7)

Return
/*************\
*   KEY_SNT   *   /* undefined */
\*************/
Get_SNT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_SNT_char_name   +4,
     KEY_SNT_char_filler +8,
     DAT_SNT_char_filler +4
Return
/*************\
*   KEY_TCT   *                  /* undefined */
\*************/
Get_TCT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_TCT_char_name   +4,
     KEY_TCT_char_filler +8,
     DAT_TCT_char_filler +4
Return
/*************\
*   KEY_TCS   *   /* what is this code? */
\*************/
Get_TCS_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_TCS_char_name   +4,
     KEY_TCS_char_filler +8,
     DAT_TCS_char_filler +4
Return
/*************\
*   KEY_TST   *                           /* undefined */
\*************/
Get_TST_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_TST_char_name   +4,
     KEY_TST_char_tran   +8,
     DAT_TST_char_filler +4
Return
/*************\
*   KEY_DCT   *                     /* undefined */
\*************/
Get_DCT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_DCT_char_name   +4,
     .                   +8,
     KEY_DCT_char_tran   +4,
     DAT_DCT_char_filler
Return
/*************\
*   KEY_PPT   *                   /* undefined */
\*************/
Get_PPT_Type_Record:
  Arg commarea
  Parse Value ParseCommarea() with,
     KEY_PPT_char_name         +8,
     KEY_PPT_char_tran         +4,
     DAT_PPT_char_ctw_ppt_type +1, /*--> CTW_PPT_PROGRAM      'P' */
     DAT_PPT_char_ctw_ppt_res  +1, /*--> CTW_PPT_PERMANENT    'P' */
     .
Return
/*************\
*   KEY_PID   *                   /* OS/2 PID Processer Stats */
\*************/
Get_PID_Type_Record:
  Arg commarea
  pidprogram.  = ''                          /* set array to nulls       */
  Parse Value ParseCommarea() with,
     KEY_PID_char_name               +2,     /* Process ID               */
     .                               +10,    /* filler                   */
     DAT_PID_ulong_pidnotran         +4,     /* Num of Trans Run         */
     DAT_PID_ulong_pidtotaltime      +4,     /* Total time for           */
     DAT_PID_ulong_pidacttotaltime   +4,     /* Active Total             */
     DAT_PID_ulong_pidtaskn          +4,     /* Task number              */
     DAT_PID_short_pidprty           +2,     /* Task Priority            */
     DAT_PID_short_pidprogramno      +2,     /* Depth of programs        */
     DAT_PID_char_pidprogram.1       +8,     /* up to five               */
     DAT_PID_char_pidprogram.2       +8,     /* program                  */
     DAT_PID_char_pidprogram.3       +8,     /* names                    */
     DAT_PID_char_pidprogram.4       +8,     /* can be                   */
     DAT_PID_char_pidprogram.5       +8,     /* saved                    */
     DAT_PID_char_pidprogram.6       +8,     /* saved                    */
     DAT_PID_char_pidprogram.7       +8,     /* saved                    */
     DAT_PID_char_pidprogram.8       +8,     /* saved                    */
     DAT_PID_char_pidprogram.9       +8,     /* saved                    */
     DAT_PID_char_pidtran            +4,     /* TranID                   */
     DAT_PID_unsigned_pidglbl        +1      /* Global Flag              */

     pid_name        = Z2D(KEY_PID_char_name)
     pidnotran       = Z2D(DAT_PID_ulong_pidnotran)
     pidtotaltime    = ftime(Z2D(DAT_PID_ulong_pidtotaltime))
     pidacttotaltime = ftime(Z2D(DAT_PID_ulong_pidacttotaltime))
     pidtaskn        = Z2D(DAT_PID_ulong_pidtaskn)   /*Task number offset */
     pidprty         = Z2D(DAT_PID_short_pidprty)      /* convert to      */
     pidprty         = trunc(((pidprty/256)+.005),2)   /* OS/2 Priority   */
     pidprogramno    = Z2D(DAT_PID_short_pidprogramno)
     do P=1 to 9                             /* Extract Program Names     */
        pidprogram.P = DAT_PID_char_pidprogram.P
        if pidprogram.P = '0000000000000000'x Then pidprogram.P=''
     end
     pidtran         = DAT_PID_char_pidtran
     pidglbl         = DAT_PID_unsigned_pidglbl      /* Global Process    */
     if pidtran||pidglbl='0000000000'x Then pidglbl='GLBL'
     else pidglbl=''
     if pidtran      = '00000000'x Then pidtran=''   /* cvt 00 to blanks  */
     Line.I = center(pid_name,6) right(pidnotran,6) right(pidtotaltime,8),
              right(pidacttotaltime,8) right(pidprty,7) right(pidtran,4),
              right(pidtaskn,4),
              right(pidprogramno,5),
              pidglbl||pidprogram.pidprogramno
Return
/*-------------------------------------------------------------*/
/* generalized Error Routine                                   */
/*-------------------------------------------------------------*/
error:
  arg erc message
  say 'error' erc message
  exit erc
return
/*-------------------------------------------------------------*/
/* Subdivide Commarea   Records are 136 bytes long             */
/*-------------------------------------------------------------*/
ParseCommarea: Procedure Expose commarea
  Parse Value commarea with 1 first +136 commarea
Return First
/*-------------------------------------------------------------*/
/* REXX routines to convert formatted numbers to REXX decimals */
/*-------------------------------------------------------------*/
/* ================================================================= */
/*  Z2d - Return decimal value of Intel packed value                 */
/* ================================================================= */
Z2d:
  parse arg Z
  /* this is a SHORT or INT */
  if length(Z) = 2 then
    return strip(right(c2d(reverse(Z)),8))

  /* this is LONG int */
  if length(Z) = 4 & substr(Z,4,1) < '3A'x then do
    x = translate('DCBA',Z,'ABCD')
    x = right(c2d(x), 8)
    return strip(x)
  end

Return '0'
/* ================================================================= */
/*  Double - Return the decimal value of an IEEE float number        */
/* ================================================================= */
Double:
  trace o
  parse arg float
  double = reverse(float)
  bin = x2b(c2x(float))
  if (length(bin) = 4*8) then do /* 4 bytes for a normal float */
    bsign = substr(bin, 1, 1)  /* get the sign bit */
    bexp =  substr(bin, 2, 8)  /* get the exponent bits */
    bmant = substr(bin, 10)    /* get the mantissa bits */
  end
  else do /* this is a DOUBLE float */
    bsign = substr(bin, 1, 1)   /* get the sign bit */
    bexp =  substr(bin, 2, 11)  /* get the exponent bits */
    bmant = substr(bin, 13)     /* get the mantissa bits */
  end
  exp = x2d(b2x(right(bexp, 2*8, "0"))) - 1023 /* calculate real exponent */
  /* get the left part of the mantissa that corresponds to the */
  /* number that is >= 1.0 */
  blexp = left(bmant, exp)
  lexp = x2d(b2x(blexp))
  brexp = substr(bmant, exp+1)
  lexp = 2**exp + lexp
  /* use the rightmost part to calculate the decimal points */
  rexp = 0
  do i = 1 to length(brexp)
    x = substr(brexp, i, 1)
    if (x = 1) then rexp = rexp + 2**-i
  end
  number = lexp+rexp
  if (bsign = 1) then
    number = -1*number
return number
/*--------------------------------------------------------------------*/
/* Unpack a number                                                    */
/*--------------------------------------------------------------------*/
unpack:
  arg number
  number=c2x(number)
  If right(number,1)='C' Then number=left(number,length(number)-1)
return number
/*--------------------------------------------------------------------*/
/* format time, recorded in milliseconds                              */
/*--------------------------------------------------------------------*/
ftime:
  arg number
  number=right('0000000'||number,7)
  ms=right(number,3)
  secs=left(number,4)
  secs=secs+0
  number=right(secs'.'ms,8)
  number=format(number,5,2)
Return number
/*--------------------------------------------------------------------*/
/* CHECKECI     - Check the eci.return_code passed and report         */
/*--------------------------------------------------------------------*/
checkeci: Procedure expose ECI.
  Arg eci_rc .
  Select                                      /* Check of non-zero rcs */
  When eci_rc ='-1'  Then
       say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-LENGTH'
  When eci_rc ='-2'  Then
       say 'rc='eci_rc 'ECI-ERR-INVALID-EXTEND-MODE'
  When eci_rc ='-3'  Then
       say 'rc='eci_rc 'ECI-ERR-NO-CICS'
  When eci_rc ='-4'  Then
       say 'rc='eci_rc 'ECI-ERR-CICS-DIED'
  When eci_rc ='-5'  Then
       say 'rc='eci_rc 'ECI-ERR-REQUEST-TIMEOUT'
  When eci_rc ='-6'  Then
       say 'rc='eci_rc 'ECI-ERR-RESPONSE-TIMEOUT'
  When eci_rc ='-7'  Then
       Do
         say 'rc='eci_rc 'ECI-ERR-TRANSACTION-ABEND'
         say 'ECI Abend Code='eci.abend_code
       End
  When eci_rc ='-8'  Then
       say 'rc='eci_rc 'ECI-ERR-EXEC-NOT-RESIDENT or ECI-ERR-LUW-TOKEN'
  When eci_rc ='-9'  Then
       say 'rc='eci_rc 'ECI-ERR-SYSTEM-ERROR'
  When eci_rc ='-10' Then
       say 'rc='eci_rc 'ECI-ERR-NULL-WIN-HANDLE'
  When eci_rc ='-12' Then
       say 'rc='eci_rc 'ECI-ERR-NULL-MESSAGE-ID'
  When eci_rc ='-13' Then
       say 'rc='eci_rc 'ECI-ERR-THREAD-CREATE-ERROR'
  When eci_rc ='-14' Then
       say 'rc='eci_rc 'ECI-ERR-INVALID-CALL-TYPE'
  When eci_rc ='-15' Then
       say 'rc='eci_rc 'ECI-ERR-ALREADY-ACTIVE'
  When eci_rc ='-16' Then
       say 'rc='eci_rc 'ECI-ERR-RESOURCE-SHORTAGE'
  When eci_rc ='-17' Then
       say 'rc='eci_rc 'ECI-ERR-NO-SESSIONS'
  When eci_rc ='-18' Then
       say 'rc='eci_rc 'ECI-ERR-NULL-SEM-HANDLE'
  When eci_rc ='-19' Then
       say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-AREA  '
  Otherwise
       say 'rc='eci_rc 'is unknown'
  End
Return
/*-------------------------------------------------------*/
/* RexxSetup - Ensure RexxSetup is loaded                */
/*-------------------------------------------------------*/
RexxSetup: Procedure
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0
/*-------------------------------------------------------*/
/* EciSetup - Ensure RxCICS is loaded                    */
/*-------------------------------------------------------*/
EciSetup: Procedure
  If rxfuncquery('ECI') Then Do
     If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then signal failure
     say "ECI registered"
  End
Return 0
/*-------------------------------------------------------*/
/* Check the Return Code from the ECI call               */
/*-------------------------------------------------------*/
check_rc: procedure  expose ECI. DEBUG
  If DEBUG>0 Then rc = show_eci_stem()   /* show call options */
  If ECI.return_code<>0 Then CALL CHECKECI eci.return_code
return(0)
/*-------------------------------------------------------*/
/*  Show the ECI. stem values in debug mode              */
/*-------------------------------------------------------*/
show_eci_stem: procedure  expose ECI.
  say "Contents of ECI stem:"
  say "      call_type         : " ECI.call_type
  say "      program name      : " ECI.program_name
  say "      userid            : " ECI.userid
  say "      password          : " ECI.password
  say "      transid           : " ECI.transid
  say "      abend_code        : " ECI.abend_code
  say "      commarea_length   : " ECI.commarea_length
  say "      timeout           : " ECI.timeout
  say "      sys_return_code   : " ECI.sys_return_code
  say "      extended_mode     : " ECI.extend_mode
  say "      sem_handle        : " ECI.ASYNC_NOTIFY.SEM_HANDLE
  say "      luw_token         : " ECI.luw_token
  say "      return code       : " ECI.return_code
  '@pause'
return(0)
/*----------------------------------------*/
/* called if something fails              */
/*----------------------------------------*/
FAILURE:
  Say 'Failure to add function at line' sigl
  "@pause"
Return 1
/*--------------------End of CALLPA2.CMD-----------------*/
