/* TECHD.CMD   */
/*******************************************************************/
/*                                                                 */
/* MODULE NAME      TECHD.CMD                                      */
/*                                                                 */
/* DESCRIPTIVE NAME Use RxECI to the VSAMSERV application          */
/*                                                                 */
/* Statement:       Licensed Materials - Property of IBM           */
/*                                                                 */
/*                  CA31 SupportPac                                */
/*                  (c) Copyright IBM Corp. 1988, 1994.            */
/*                                                                 */
/*                  See Copyright Instructions.                    */
/*                                                                 */
/*                  All rights reserved.                           */
/*                                                                 */
/*                  U.S. Government Users Restricted Rights - use, */
/*                  duplication or disclosure restricted by GSA    */
/*                  ADP Schedule Contract with IBM Corp.           */
/*                                                                 */
/* Status:          Version 2 Release 2                            */
/* Modified         19th May 1994                                  */              
/*                  Mean calculation modified to reduce overhead   */
/*                                                                 */
/*  NOTES :-                                                       */
/*    DEPENDENCIES = OS/2 2.x, CICS OS/2, RxECI                    */
/*                   None                                          */
/*    RESTRICTIONS = none                                          */
/*    MODULE TYPE  = Command file                                  */
/*    PROCESSOR    = PS/2 and PC                                   */
/*                                                                 */
/*******************************************************************/
/*******************************************************************/
/*                                                                 */
/*  Please see the prolog for the TESTRUN.CMD for additional       */
/*  information.                                                   */
/*                                                                 */
/*  This is the front-end that uses a back-end program called      */
/*  VSAMSERV along with a VSAM file, TECHBASE.  In addition,       */
/*  it uses the ECHO back-end program.                             */
/*  It possible to use TESTRUN with a mainframe CICS system. To do */
/*  this you will need to the equivalent to VSAMSERV on the CICS   */
/*  host and a TECHBASE VSAM dtabase.  Rather than create yourself */
/*  you are advised to contact the TS Demo team in Hursley (via    */
/*  the TSDEMO FORUM at WINVMC) who can arrange for access to a    */
/*  CICS MVS host in Warwick.                                      */
/*                                                                 */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*  The VSAMSERV.DLL must be available to CICS OS/2.  Also, the    */
/*  TECHBASE.BTR file must be defined and available.  These can    */
/*  be easily defined using CEDA (PPT for the VSAMSERV program and */
/*  FCT for the TECHBASE and TECHALT definitions).  You may want   */
/*  to consider defining VSAMSERV (and ECHO) as Resident programs, */
/*  but that is not required.  The TECHBASE and TECHALT (the       */
/*  alternate index) definition information is as follows:         */
/*                                                                 */
/*   File name  ............ TECHBASE   Group Name .......... Demo */
/*   Dataset name .......... DATA\TECHBASE.BTR                     */
/*   File open ............. Y                                     */
/*   File enabled .......... Y                                     */
/*   File type ............. K                                     */
/*   File Access Method .... U                                     */
/*   Base File Name ........         Key Number .............      */
/*   Minimum Record length . 0020    Maximum Record length .. 00100*/
/*   CI Size ............... 0512    External File Manager .. N    */
/*   System ID .............         Remote File Name .......      */
/*   Description ........... Demo base File                        */
/*                                                                 */
/*   Key Segments                                                  */
/*        Pos  Len  Dup  Mod  Bin  Nc  Sg  Alt  Num   Key Num      */
/*   01   000  005   N    N    N    N   N   N   000                */
/*   02   005  015   N    N    N    N   N   N   000                */
/*   03                                                            */
/*   04                                                            */
/*                                                                 */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*                                                                 */
/*   File name  ............ TECHALT    Group Name .......... Demo */
/*   Dataset name .......... DATA\TECHBASE.BTR                     */
/*   File open ............. Y                                     */
/*   File enabled .......... Y                                     */
/*   File type ............. A                                     */
/*   File Access Method .... U                                     */
/*   Base File Name ........         Key Number ............. 01   */
/*   Minimum Record length .         Maximum Record length ..      */
/*   CI Size ...............         External File Manager .. N    */
/*   System ID .............         Remote File Name .......      */
/*   Description ........... Demo Alt Index                        */
/*                                                                 */
/*   Key Segments                                                  */
/*        Pos  Len  Dup  Mod  Bin  Nc  Sg  Alt  Num   Key Num      */
/*   01                                                            */
/*   02                                                            */
/*   03                                                            */
/*   04                                                            */
/*                                                                 */
/*-----------------------------------------------------------------*/
/*                                                                 */
/*  The design of this function is to make repeated calls for      */
/*  the following:                                                 */
/*                                                                 */
/*  1. Read one of the records (based on a randomly generated key) */
/*  2. Wait a random number of seconds                             */
/*  3. Update the record read                                      */
/*  4. Wait a random number of seconds                             */
/*  5. Make a call to the ECHO program                             */
/*  6. Wait a random number of seconds                             */
/*                                                                 */
/*  It also uses a random amount of wait time, between 1 and       */
/*  10 seconds.  The upper bound on this is defined by the         */
/*  MAXSLEEP value shown below.                                    */
/*                                                                 */
/*-----------------------------------------------------------------*/
DEBUG = 0                                 /* Used by RxECI         */
  MAXSLEEP  = 10                          /* Max sleep time        */
  Arg log_file .                          /* Name of the Log File  */
  Call SysCurPos '1', '0'                 /* Position screen       */
  Call RexxSetup                          /* Ensure RexxUtil loaded*/
  Call SysCls                             /* Clear the screen      */
  Call EciSetup                           /* Ensure RxCICS loaded  */
  Parse Value '0 0 0 0 0 0 0' With mean_count high i bs t log sleep log_id response_list
  If log_file<>'' Then                    /* Get LogFile Name      */
    Do
      logname=Filespec('name',log_file)
      logname=Substr(logname,1,Lastpos('.',logname)-1)
      log_id=Left(logname,8)
      log=1
      Say 'Logging to:' log_file
    End
  started=Time() 'on' Date()              /* When started          */
  Say 'TECHD starting at' started         /* Tell the user         */
  If Log Then '@ECHO' log_id 'started at' started '>>' log_file
  sleep=0
  Do Forever                              /* Run forever           */
    ECI.program_name = "VSAMSERV"         /* CICS OS/2 PPT program */
    Call SysCurPos '3', '0'               /* Position screen       */
    t=t+1                                 /* Times run             */
    /*-------------------------------------------------------------*/
    /* The sample TECHBASE.BTR contains records with keys ranging  */
    /* from 00001 to 00094.  For fewer or more records, change the */
    /* Random(1,94) statement below as needed.                     */
    /*--------------Key Range--------------------------------------*/
    key=Right('0000'Random(1,94),5)       /* generate a random key */
    commareaR= Left('2005'key,140)        /* Commarea is FIXED Size*/
    CALI=Length(commareaR)                /* but record it anyway  */
    ECI.commarea_length = CALI            /* set the length        */
    Say 'Reading Record ('key') at' time() /* Information message  */
    a=Time('R')                           /* Start the timer       */
    commarea2 = ECI('ECI.',commareaR)     /* Make the READ call    */
    b=Time('E')                           /* Stop the Timer        */
    Say Substr(commarea2,1,78)            /* display results       */
    If Substr(commarea2,81)<>'' Then      /* If any trailing msgs  */
       Say Substr(commarea2,81)           /* then display them     */
    Call ecirc                            /* Check the r/c         */
    If ECI.return_code<>0 Then Leave      /* Run till non-zero     */
    sleep=Random(1,MAXSLEEP)              /* sleeping time         */
    Say 'Waiting for' sleep 'seconds'     /* information message   */
/*----------------------------------------   Next, do an Update    */
    Call SysCurPos '10', '0'              /* Position screen       */
    Call SysSleep sleep                   /* sleep 1 to 10 seconds */
    t=t+1                                 /* count again           */
    Say 'Updating Record ('key') at' time() /* Information message */
    a=Time('R')                           /* start the timer       */
    commarea3 = ECI('ECI.','3'substr(commarea2,2,139)) /* UPDATE Call*/
    b=Time('E')                           /* stop the timer        */
    Say Substr(commarea3,1,78)            /* display results       */
    If Substr(commarea3,81)<>'' Then      /* If any trailing msgs  */
       Say Substr(commarea3,81)           /* then display them     */
    Call ecirc                            /* Check the r/c         */
    If ECI.return_code<>0 Then Leave      /* Run till non-zero     */
    sleep=Random(1,MAXSLEEP)              /* sleeping time         */
    Say 'Waiting for' sleep 'seconds'     /* information message   */
    Call SysSleep sleep                   /* sleep 1 to 10 seconds */
/*----------------------------------------   call the Echo Program */
    Call SysCurPos '18', '0'              /* Position screen       */
    ECI.program_name = "ECHO"             /* PPT program name      */
    t=t+1                                 /* count again           */
    commareaE=left('EchoTest',50)         /* new commarea length   */
    CALI=Length(commareaE)                /* for the echo program  */
    ECI.commarea_length = CALI            /* set the length        */
    Say 'Running the Echo program at' time() /* information message*/
    a=Time('R')                           /* start the timer       */
    commarea4 = ECI('ECI.',commareaE)     /* make the echo call    */
    b=Time('E')                           /* stop the timer        */
    Say commarea4                         /* record the results    */
    call ecirc                            /* Check the r/c         */
    If ECI.return_code<>0 Then Leave      /* Run till non-zero     */
    sleep=Random(1,MAXSLEEP)              /* sleeping time         */
    Call SysSleep sleep                   /* sleep 1 to 10 seconds */
    Say 'Waiting for' sleep 'seconds'     /* information message   */
  End                                     /* End of Do Forever     */
  If Log Then                             /* Log the termination   */
    '@ECHO' log_id 'stopped at' time() date() date('D') '>>' log_file
  '@EXIT'                                 /* Terminate the window  */
Return 0                                  /* Never executed        */

FAILURE:
  Say 'Failure to add function at line' sigl
  Return 1
/* -------ECI-ERROR-ID---------------------------------------------*/
ecirc:
    If b>high Then high=b                 /* Save High time        */
    bs=bs+b                               /* total time used       */
    avg=Format((bs/t),2,3)                /* format the average    */
    last=Format(b,2,3)                    /* format the last time  */
    meant=mean(last)                      /* get the mean value    */
    high=Format(high,2,3)                 /* format the high time  */
    information= 'Avg ='avg,              /* report average time   */
                 'Last='last,             /* and last time         */
                 'mean='meant,            /* Mean time             */
                 'sleep=' sleep,          /* sleep (wait)          */
                 'longest=' high          /* longest time          */
    Say Left(information,79)              /* Information Message   */
    If Log Then                           /* Log the results       */
      Do
         log_rec=log_id Left(ECI.program_name,8),
         Right(t,8) Left(avg,6) Left(last,6) Left(meant,6) Right(sleep,2),
         Left(high,6) Right(CALI,4) Right(ECI.return_code,3),
         Time() Date() ECI.abend_code
         '@ECHO' log_rec '>>' log_file
      End
    If ECI.return_code ='0'   Then        /* if good, goback       */
      Do
         say 'rc='ECI.return_code 'ECI-NO-ERROR'
         Return
      End
    CALL CHECKECI ECI.return_code         /* else report it        */
Return
/*------------------------------------------------------------------------*/
/* Compute the mean response time for 500 values only to minimize overhead*/
/*------------------------------------------------------------------------*/
Mean: Procedure Expose response_list mean_count
  Arg resp .                                 /*                           */
  new_list=''                                /*                           */
  mean_count=mean_count+1                    /* count the number of values*/
  If mean_count>499 Then                     /* use up to 500 values      */
    Do                                       /* to kee[ down overhead     */
      mean_count=0                           /* reset the count           */
      response_list=''                       /* and the list              */
    End                                      /*                           */
  If response_list='' Then                   /*                           */
    Do                                       /*                           */
      response_list=resp                     /*                           */
      Return resp                            /*                           */
    End                                      /*                           */
  Count=words(response_list)                 /*                           */
  first_word=word(response_list,1)           /*                           */
  last_word =word(response_list,Count)       /*                           */
  If resp<first_word Then                    /*                           */
    Do                                       /*                           */
      response_list=resp response_list       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */
  If resp> last_word Then                    /*                           */
    Do                                       /*                           */
      response_list=response_list resp       /*                           */
      m=words(response_list)                 /*                           */
      n=m%2                                  /*                           */
      if n=0 Then n=1                        /*                           */
      mean=word(response_list,n)             /*                           */
      Return mean                            /*                           */
    End                                      /*                           */

  Do R=1 to Count                            /*                           */
     here=word(response_list,R)              /*                           */
     Select                                  /*                           */
     When resp>here Then                     /*                           */
       do                                    /*                           */
         new_list=new_list here              /*                           */
       End                                   /*                           */
     When resp<here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     When resp=here Then                     /*                           */
       Do                                    /*                           */
         new_list=new_list resp subword(response_list,R)
         response_list=new_list              /*                           */
         m=words(response_list)              /*                           */
         n=m%2                               /*                           */
         if n=0 Then n=1                     /*                           */
         mean=word(response_list,n)          /*                           */
         Return mean                         /*                           */
       End                                   /*                           */
     Otherwise                               /*                           */
     End                                     /*                           */
  End                                        /*                           */
  response_list=new_list                     /*                           */
  m=words(response_list)                     /*                           */
  n=m%2                                      /*                           */
  if n=0 Then n=1                            /*                           */
  mean=word(response_list,n)                 /*                           */
Return mean                                  /*                           */
/*-----------------------------------------------------------------*/
/* RexxSetup - Ensure RexxSetup is loaded                          */
/*-----------------------------------------------------------------*/
RexxSetup: Procedure
  Call RxFuncAdd 'SysLoadFuncs', 'RexxUtil', 'SysLoadFuncs'
  Call SysLoadFuncs
Return 0
/*-----------------------------------------------------------------*/
/* EciSetup - Ensure RxCICS is loaded                              */
/*-----------------------------------------------------------------*/
EciSetup: Procedure
  If rxfuncquery('ECI') Then
    Do
      If rxfuncadd('ECI', 'RXECI', 'RxECI' ) Then signal failure
      say "ECI registered"
    End
Return 0
/*--------------------------------------------------------------------*/
/* CHECKECI     - Check the eci.return_code passed and report         */
/*--------------------------------------------------------------------*/
checkeci: Procedure
Arg eci_rc .
  Select                                      /* Check of non-zero rcs */
  When eci_rc ='-1'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-LENGTH'
  When eci_rc ='-2'  Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-EXTEND-MODE'
  When eci_rc ='-3'  Then
      say 'rc='eci_rc 'ECI-ERR-NO-CICS'
  When eci_rc ='-4'  Then
      say 'rc='eci_rc 'ECI-ERR-CICS-DIED'
  When eci_rc ='-5'  Then
      say 'rc='eci_rc 'ECI-ERR-REQUEST-TIMEOUT'
  When eci_rc ='-6'  Then
      say 'rc='eci_rc 'ECI-ERR-RESPONSE-TIMEOUT'
  When eci_rc ='-7'  Then
    Do
      say 'rc='eci_rc 'ECI-ERR-TRANSACTION-ABEND'
      say 'ECI Abend Code='eci.abend_code
    End
  When eci_rc ='-8'  Then
      say 'rc='eci_rc 'ECI-ERR-EXEC-NOT-RESIDENT or ECI-ERR-LUW-TOKEN'
  When eci_rc ='-9'  Then
      say 'rc='eci_rc 'ECI-ERR-SYSTEM-ERROR'
  When eci_rc ='-10' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-WIN-HANDLE'
  When eci_rc ='-12' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-MESSAGE-ID'
  When eci_rc ='-13' Then
      say 'rc='eci_rc 'ECI-ERR-THREAD-CREATE-ERROR'
  When eci_rc ='-14' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-CALL-TYPE'
  When eci_rc ='-15' Then
      say 'rc='eci_rc 'ECI-ERR-ALREADY-ACTIVE'
  When eci_rc ='-16' Then
      say 'rc='eci_rc 'ECI-ERR-RESOURCE-SHORTAGE'
  When eci_rc ='-17' Then
      say 'rc='eci_rc 'ECI-ERR-NO-SESSIONS'
  When eci_rc ='-18' Then
      say 'rc='eci_rc 'ECI-ERR-NULL-SEM-HANDLE'
  When eci_rc ='-19' Then
      say 'rc='eci_rc 'ECI-ERR-INVALID-DATA-AREA  '
  Otherwise
      say 'rc='eci_rc 'is unknown'
End
Return
/*----------------------------- End of Code--------------------*/
                                                   